#include "AT898252.H"
#include "1Wire.h"
#include "service.h"
#include "lcd.h"
#include "ACL.h"
#include "defines.h"
#include "time.h"
#include "dataflash.h"

extern void send_byte(char,char);
extern char receive_byte(char);
extern void _opendoor(void);

extern char DO_CRC(char oldCRC,char next);
extern char iData[8];
extern char sz[17];
extern char cr;
extern char i;
extern time_t internalTime;

#define iPort P0

// Convert byte chain to HEX string
void Byte2Hex (char *dest,char *src, char srclen)
{
	char temp;
	char cur=0;
	char i=0;

	do
	{
		// HI part
		temp=(src[i]>>4)&0x0F;
		if (temp<=9)
			dest[cur++]='0'+temp;
		else
			dest[cur++]='A'+temp-10;

		// LO part
		temp=src[i]&0x0F;
		if (temp<=9)
			dest[cur++]='0'+temp;
		else
			dest[cur++]='A'+temp-10;
		
		i++;
	}
	while (i<srclen);
}

// Query TouchPad
char QueryTouch (void)
{
	sr xdata *pSR=0x20;
		
	unsigned char j;
	char fl=0;

	WMCON|=WDTRST_;  // reset watchdog

	if (OneWireReadSerial())
	{
//copy iData to sz,because when we read time from RTC iData contents are rewritten
		sz[0]=iData[0];
		sz[1]=iData[1];
		sz[2]=iData[2];
		sz[3]=iData[3];
		sz[4]=iData[4];
		sz[5]=iData[5];
		sz[6]=iData[6];
		sz[7]=iData[7];

		WMCON=WMCON|EEMEN_;
	
		// Look through memory slots and find matching one
		j=-1;
		fl=1;
cont2:
		GetCurTime();

		while (++j<MAXSR)
		{
			if (pSR[j].type==0 || pSR[j].type==0xFF)	// if record is marked as deleted - skip it
				goto cont2;
			fl=0;

			for (cr=0;cr<7; cr++)
				if (pSR[j].sid[cr]!=sz[cr+1])
					goto cont2;				// Key does not match

			// if we have reached this place, this means that the key is found !
			if (pSR[j].type==3)	// check whether login has expired
				if (pSR[j].timeLimit<internalTime)
					goto cont2;
			///////////HERE ALL ACTIONS, WHEN DOOR OPENS
			//write SID
			df_write_through_buffer1(sz,8);
			//Write access time
			df_write_through_buffer1(&internalTime,4);
			opendoor();
			return pSR[j].type;
		}
	}
	return fl;
}

// Wait until 1-wire line becomes LOW (Can be used to measure HI-state in MIPS)
char getHiStateTime (char pin)
{
	i=0;
	do				// one pass is 3 mks. Max wait 3*255=765mks
		i++;
	while ((iPort&pin) && i);

	return i;
}
// Wait until 1-wire line becomes HI (Can be used to measure LOW-state in MIPS)
char getLowStateTime (char pin)
{
	i=0;
	do				// one pass is 3 mks. Max wait 3*255=765mks
		i++;		
	while ((iPort&pin)==0 && i);

	return i;
}

// Send reset pulse
void OneWireReset(char pin)
{
	char len;

	len=160;
	iPort&=~pin;			// reset pulse

	while(len--);

	iPort|=pin;			// end of reset pulse
}

// it first waits until iButton startts to transmit presense and then waits
// until the transmission is over
char OneWireReadPresense(char pin)
{

	i=getHiStateTime(pin);	// wait for presense pulse
	return i && getLowStateTime(pin);	// wait for end of presense pulse;
}

// makes one try to read serial number from 1Wire bus
char OneWireReadSerial(void)
{
	char crc;

	OneWireReset(2);
	if (OneWireReadPresense(2))
	{
		send_byte(51,2);// read ROM

		i=7;
		while(i>=0)
			iData[i--]=_receive_byte(2);

		crc=0;
		i=7;
		while (i>=1)
			crc=DO_CRC(crc,iData[i--]);
		
		if (!iData[0])
			return 0; // CRC is 0 - error !

		if (iData[0]==crc)
			return 1;
	}

	return 0;
}

// read memory contents from iButton
char OneWireReadMem(int addr,char* dest,int size)
{
	char *paddr;
	int i=0;
	OneWireReset(1);
	if (OneWireReadPresense(1))
	{
		// skip rom
		send_byte(0xCC,1);
		// read mem command
		send_byte(0xF0,1);
		paddr=&addr;
		// low addr byte
		send_byte(*(paddr+1),1);
		// hi addr byte
		send_byte(*paddr,1);
		
		while (i<size)
			dest[i++]=_receive_byte(1);

		OneWireReset(1);
		return 1;
	}

	return 0;
}

// Write to iButton memory
char OneWireWriteMem(int addr,char* src,char size)
{
	char stat; // status register state

	// send data to scratchpad
	OneWireReset(1);
	OneWireReadPresense(1);
	send_byte(0xCC,1);
	OneWireWriteScratchpad(addr,src,size);

	// verify data from scratchpad
	OneWireReset(1);
	OneWireReadPresense(1);
	send_byte(0xCC,1);
	if (!OneWireReadScratchpad(addr,&stat,src,size))
		return 0;

	// send copy scratchpad command
	OneWireReset(1);
	OneWireReadPresense(1);
	send_byte(0xCC,1);
	OneWireCopyScratchpad(addr, stat);

	OneWireReset(1);
	while (OneWireReadPresense(1)==0) // wait untul copy scratchpad is finished
		OneWireReset(1);
}

// Write to temporary buffer of iButton
void OneWireWriteScratchpad(int addr, char *src, char size)
{
	char *paddr;
	register char i=0;

	paddr=&addr;

	send_byte(0x0F,1);		// write scratchpad command
	send_byte(*(paddr+1),1); 	// low addr byte
	send_byte(*paddr,1); 		// hi addr byte
	while (i<size)			// send data		
		send_byte(src[i++],1);
}
// Read from temporary buffer of iButton
char OneWireReadScratchpad(int addr, char* stat,char *src, char size)
{
	char *paddr;
	char rcv;
	register char i=0;

	paddr=&addr;
	send_byte(0xAA,1);		// read scratchpad command

	rcv=_receive_byte(1); 	// low addr byte
	if (rcv!=*(paddr+1))
		return 0;

	rcv=_receive_byte(1); 	// hi addr byte
	if (rcv!=*paddr)
		return 0;

	*stat=_receive_byte(1);	// receive status register state

	while (i<size)			// receive data
		if (_receive_byte(1)!=src[i++])
			return 0;

	return 1;	
}

// Make iButton update it's RAM from Scratchpad
void OneWireCopyScratchpad(int addr, char stat)
{
	char *paddr;

	paddr=&addr;

	send_byte(0x55,1);		// copy scratchpad command
	send_byte(*(paddr+1),1); 	// low addr byte
	send_byte(*paddr,1); 		// hi addr byte
	send_byte(stat,1);		// send status register state
}
